@@ -5,7 +5,7 @@
  *
  * Part:        Healthcheckers dynamic data structure definition.
  *
- * Version:     $Id: check_data.h,v 1.1.12 2006/03/09 01:22:13 acassen Exp $
+ * Version:     $Id: check_data.h,v 1.1.15 2007/09/15 04:07:41 acassen Exp $
  *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
  *
@@ -19,7 +19,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2006 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2007 Alexandre Cassen, <acassen@freebox.fr>
  */
 
 #ifndef _CHECK_DATA_H
@@ -75,6 +75,10 @@ typedef struct _real_server {
 	uint32_t addr_ip;
 	uint16_t addr_port;
 	int weight;
+#ifdef _KRNL_2_6_
+	uint32_t u_threshold;   /* Upper connection limit. */
+	uint32_t l_threshold;   /* Lower connection limit. */
+#endif
 	int inhibit;		/* Set weight to 0 instead of removing
 				 * the service from IPVS topology.
 				 */
