@@ -5,7 +5,7 @@
  *
  * Part:        NETLINK IPv4 routes manipulation.
  *
- * Version:     $Id: vrrp_iproute.c,v 1.1.12 2006/03/09 01:22:13 acassen Exp $
+ * Version:     $Id: vrrp_iproute.c,v 1.1.15 2007/09/15 04:07:41 acassen Exp $
  *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
  *
@@ -19,7 +19,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2006 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2007 Alexandre Cassen, <acassen@freebox.fr>
  */
 
 /* local include */
@@ -64,6 +64,8 @@ netlink_route_ipv4(ip_route *iproute, int cmd)
 		addattr32(&req.n, sizeof(req), RTA_OIF, iproute->index);
 	if (iproute->src)
 		addattr_l(&req.n, sizeof(req), RTA_PREFSRC, &iproute->src, 4);
+	if (iproute->metric)
+		addattr32(&req.n, sizeof(req), RTA_PRIORITY, iproute->metric);
 
 	if (netlink_talk(&nl_cmd, &req.n) < 0)
 		status = -1;
@@ -103,7 +105,7 @@ void
 dump_iproute(void *rt_data_obj)
 {
 	ip_route *route = rt_data_obj;
-	char *log_msg = MALLOC(100);
+	char *log_msg = MALLOC(150);
 	char *tmp = MALLOC(30);
 
 	if (route->dst) {
@@ -132,6 +134,10 @@ dump_iproute(void *rt_data_obj)
 			 netlink_scope_n2a(route->scope));
 		strncat(log_msg, tmp, 30);
 	}
+	if (route->metric) {
+		snprintf(tmp, 30, " metric %d", route->metric);
+		strncat(log_msg, tmp, 30);
+	}
 
 	syslog(LOG_INFO, "     %s", log_msg);
 
@@ -170,6 +176,8 @@ alloc_route(list rt_list, vector strvec)
 			new->index = IF_INDEX(ifp);
 		} else if (!strcmp(str, "table")) {
 			new->table = atoi(VECTOR_SLOT(strvec, ++i));
+		} else if (!strcmp(str, "metric")) {
+			new->metric = atoi(VECTOR_SLOT(strvec, ++i));
 		} else if (!strcmp(str, "scope")) {
 			new->scope = netlink_scope_a2n(VECTOR_SLOT(strvec, ++i));
 		} else {
