@@ -5,7 +5,7 @@
  *
  * Part:        Sheduling framework for vrrp code.
  *
- * Version:     $Id: vrrp_scheduler.c,v 1.1.12 2006/03/09 01:22:13 acassen Exp $
+ * Version:     $Id: vrrp_scheduler.c,v 1.1.15 2007/09/15 04:07:41 acassen Exp $
  *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
  *
@@ -19,7 +19,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2006 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2007 Alexandre Cassen, <acassen@freebox.fr>
  */
 
 #include "vrrp_scheduler.h"
@@ -33,6 +33,7 @@
 #include "vrrp_index.h"
 #include "ipvswrapper.h"
 #include "memory.h"
+#include "notify.h"
 #include "list.h"
 #include "main.h"
 #include "smtp.h"
@@ -70,6 +71,11 @@ static void vrrp_goto_master(vrrp_rt *);
 static void vrrp_master(vrrp_rt *);
 static void vrrp_fault(vrrp_rt *);
 
+static int vrrp_update_priority(thread * thread_obj);
+static int vrrp_script_child_timeout_thread(thread * thread_obj);
+static int vrrp_script_child_thread(thread * thread_obj);
+static int vrrp_script_thread(thread * thread_obj);
+
 struct {
 	void (*read) (vrrp_rt *, char *, int);
 	void (*read_to) (vrrp_rt *);
@@ -180,7 +186,49 @@ vrrp_init_state(list l)
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vrrp = ELEMENT_DATA(e);
 
-		if (vrrp->priority == VRRP_PRIO_OWNER ||
+		/* In case of VRRP SYNC, we have to carefully check that we are
+		 * not running floating priorities on any VRRP instance.
+		 */
+		if (vrrp->sync) {
+			element e;
+			tracked_sc *sc;
+			tracked_if *tip;
+			int warning = 0;
+
+			if (!LIST_ISEMPTY(vrrp->track_ifp)) {
+				for (e = LIST_HEAD(vrrp->track_ifp); e; ELEMENT_NEXT(e)) {
+					tip = ELEMENT_DATA(e);
+					if (tip->weight) {
+						tip->weight = 0;
+						warning++;
+					}
+				}
+			}
+
+			if (!LIST_ISEMPTY(vrrp->track_script)) {
+				for (e = LIST_HEAD(vrrp->track_script); e;
+				     ELEMENT_NEXT(e)) {
+					sc = ELEMENT_DATA(e);
+					if (sc->weight) {
+						sc->weight = 0;
+						sc->scr->inuse--;
+						warning++;
+					}
+				}
+			}
+
+			if (warning > 0) {
+				syslog(LOG_INFO, "VRRP_Instance(%s) : ignoring "
+						 "track weights due to SYNC group",
+				       vrrp->iname);
+			}
+		} else {
+			/* Register new priority update thread */
+			thread_add_timer(master, vrrp_update_priority,
+					 vrrp, vrrp->adver_int);
+		}
+
+		if (vrrp->base_priority == VRRP_PRIO_OWNER ||
 		    vrrp->wantstate == VRRP_STATE_MAST) {
 #ifdef _HAVE_IPVS_SYNCD_
 			/* Check if sync daemon handling is needed */
@@ -233,6 +281,28 @@ vrrp_init_sands(list l)
 	}
 }
 
+/* if run after vrrp_init_state(), it will be able to detect scripts that
+ * have been disabled because of a sync group and will avoid to start them.
+ */
+static void
+vrrp_init_script(list l)
+{
+	vrrp_script *vscript;
+	element e;
+
+	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+		vscript = ELEMENT_DATA(e);
+		if (vscript->inuse == 0)
+			vscript->result = VRRP_SCRIPT_STATUS_DISABLED;
+
+		if (vscript->result == VRRP_SCRIPT_STATUS_INIT) {
+			vscript->result = VRRP_SCRIPT_STATUS_NONE;
+			thread_add_timer(master, vrrp_script_thread,
+					 vscript, vscript->interval);
+		}
+	}
+}
+
 /* Timer functions */
 static TIMEVAL
 vrrp_compute_timer(const int fd)
@@ -307,6 +377,10 @@ vrrp_register_workers(list l)
 	/* Init VRRP instances sands */
 	vrrp_init_sands(vrrp_data->vrrp);
 
+	/* Init VRRP tracking scripts */
+	if (!LIST_ISEMPTY(vrrp_data->vrrp_script))
+		vrrp_init_script(vrrp_data->vrrp_script);
+
 	/* Register VRRP workers threads */
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		sock_obj = ELEMENT_DATA(e);
@@ -553,12 +627,14 @@ vrrp_leave_fault(vrrp_rt * vrrp, char *buffer, int len)
 				       vrrp->iname);
 				vrrp->state = VRRP_STATE_BACK;
 				vrrp_smtp_notifier(vrrp);
+				notify_instance_exec(vrrp, VRRP_STATE_BACK);
 			}
 		} else {
 			syslog(LOG_INFO, "VRRP_Instance(%s) Entering BACKUP STATE",
 			       vrrp->iname);
 			vrrp->state = VRRP_STATE_BACK;
 			vrrp_smtp_notifier(vrrp);
+			notify_instance_exec(vrrp, VRRP_STATE_BACK);
 		}
 	}
 }
@@ -571,8 +647,8 @@ vrrp_goto_master(vrrp_rt * vrrp)
 		syslog(LOG_INFO, "VRRP_Instance(%s) Now in FAULT state",
 		       vrrp->iname);
 		vrrp->state = VRRP_STATE_FAULT;
-		vrrp->ms_down_timer =
-		    3 * vrrp->adver_int + VRRP_TIMER_SKEW(vrrp);
+		vrrp->ms_down_timer = 3 * vrrp->adver_int + VRRP_TIMER_SKEW(vrrp);
+		notify_instance_exec(vrrp, VRRP_STATE_FAULT);
 	} else {
 		/* If becoming MASTER in IPSEC AH AUTH, we reset the anti-replay */
 		if (vrrp->ipsecah_counter->cycle) {
@@ -598,6 +674,45 @@ vrrp_gratuitous_arp_thread(thread * thread_obj)
 	return 0;
 }
 
+/* Update VRRP effective priority based on multiple checkers.
+ * This is a thread which is executed every adver_int.
+ */
+static int
+vrrp_update_priority(thread * thread_obj)
+{
+	vrrp_rt *vrrp = THREAD_ARG(thread_obj);
+	int prio_offset, new_prio;
+
+	/* compute prio_offset right here */
+	prio_offset = 0;
+
+	/* Now we will sum the weights of all interfaces which are tracked. */
+	if (!vrrp->sync && !LIST_ISEMPTY(vrrp->track_ifp))
+		 prio_offset += vrrp_tracked_weight(vrrp->track_ifp);
+
+	/* Now we will sum the weights of all scripts which are tracked. */
+	if (!vrrp->sync && !LIST_ISEMPTY(vrrp->track_script))
+		prio_offset += vrrp_script_weight(vrrp->track_script);
+
+	if (vrrp->base_priority == VRRP_PRIO_OWNER) {
+		/* we will not run a PRIO_OWNER into a non-PRIO_OWNER */
+		vrrp->effective_priority = VRRP_PRIO_OWNER;
+	} else {
+		/* WARNING! we must compute new_prio on a signed int in order
+		   to detect overflows and avoid wrapping. */
+		new_prio = vrrp->base_priority + prio_offset;
+		if (new_prio < 1)
+			new_prio = 1;
+		else if (new_prio > 254)
+			new_prio = 254;
+		vrrp->effective_priority = new_prio;
+	}
+
+	/* Register next priority update thread */
+	thread_add_timer(master, vrrp_update_priority, vrrp, vrrp->adver_int);
+	return 0;
+}
+
 static void
 vrrp_master(vrrp_rt * vrrp)
 {
@@ -671,10 +786,12 @@ vrrp_fault(vrrp_rt * vrrp)
 		vrrp_ah_sync(vrrp);
 	} else {
 		/* Otherwise, we transit to init state */
-		if (vrrp->init_state == VRRP_STATE_BACK)
+		if (vrrp->init_state == VRRP_STATE_BACK) {
 			vrrp->state = VRRP_STATE_BACK;
-		else
+			notify_instance_exec(vrrp, VRRP_STATE_BACK);
+		} else {
 			vrrp_goto_master(vrrp);
+		}
 	}
 }
 
@@ -787,3 +904,121 @@ vrrp_read_dispatcher_thread(thread * thread_obj)
 
 	return 0;
 }
+
+/* Script tracking threads */
+static int
+vrrp_script_thread(thread * thread_obj)
+{
+	vrrp_script *vscript = THREAD_ARG(thread_obj);
+	int status;
+	pid_t pid;
+
+	/* Register next timer tracker */
+	thread_add_timer(thread_obj->master, vrrp_script_thread, vscript,
+			 vscript->interval);
+
+	/* Daemonization to not degrade our scheduling timer */
+	pid = fork();
+
+	/* In case of fork is error. */
+	if (pid < 0) {
+		syslog(LOG_INFO, "Failed fork process");
+		return -1;
+	}
+
+	/* In case of this is parent process */
+	if (pid) {
+		long timeout;
+		timeout = vscript->interval;
+		thread_add_child(thread_obj->master, vrrp_script_child_thread,
+				 vscript, pid, timeout);
+		return 0;
+	}
+
+	/* Child part */
+	closeall(0);
+	open("/dev/null", O_RDWR);
+	dup(0);
+	dup(0);
+
+	/* Also need to reset the signal state */
+	{
+		sigset_t empty_set;
+		sigemptyset(&empty_set);
+		sigprocmask(SIG_SETMASK, &empty_set, NULL);
+
+		signal(SIGHUP, SIG_DFL);
+		signal(SIGINT, SIG_DFL);
+		signal(SIGTERM, SIG_DFL);
+		signal(SIGKILL, SIG_DFL);
+	}
+
+	status = system_call(vscript->script);
+
+	if (status < 0 || !WIFEXITED(status))
+		status = 0; /* Script errors aren't server errors */
+	else
+		status = WEXITSTATUS(status);
+
+	exit(status);
+}
+
+static int
+vrrp_script_child_thread(thread * thread_obj)
+{
+	int wait_status;
+	vrrp_script *vscript = THREAD_ARG(thread_obj);
+
+	if (thread_obj->type == THREAD_CHILD_TIMEOUT) {
+		pid_t pid;
+
+		pid = THREAD_CHILD_PID(thread_obj);
+
+		/* The child hasn't responded. Kill it off. */
+		vscript->result = VRRP_SCRIPT_STATUS_NONE;
+		kill(pid, SIGTERM);
+		thread_add_child(thread_obj->master, vrrp_script_child_timeout_thread,
+				 vscript, pid, 2);
+		return 0;
+	}
+
+	wait_status = THREAD_CHILD_STATUS(thread_obj);
+
+	if (WIFEXITED(wait_status)) {
+		int status;
+		status = WEXITSTATUS(wait_status);
+		if (status == 0 && vscript->result != VRRP_SCRIPT_STATUS_GOOD) {
+			syslog(LOG_INFO, "VRRP_Script(%s) succeeded", vscript->sname);
+		}
+		if (status != 0 && vscript->result != VRRP_SCRIPT_STATUS_NONE) {
+			syslog(LOG_INFO, "VRRP_Script(%s) failed", vscript->sname);
+		}
+		vscript->result = (status == 0) ?  VRRP_SCRIPT_STATUS_GOOD :
+						   VRRP_SCRIPT_STATUS_NONE;
+       }
+
+	return 0;
+}
+
+static int
+vrrp_script_child_timeout_thread(thread * thread_obj)
+{
+	pid_t pid;
+
+	if (thread_obj->type != THREAD_CHILD_TIMEOUT)
+		return 0;
+
+	/* OK, it still hasn't exited. Now really kill it off. */
+	pid = THREAD_CHILD_PID(thread_obj);
+	if (kill(pid, SIGKILL) < 0) {
+		/* Its possible it finished while we're handing this */
+		if (errno != ESRCH)
+			DBG("kill error: %s", strerror(errno));
+		return 0;
+	}
+
+	syslog(LOG_WARNING, "Process [%d] didn't respond to SIGTERM", pid);
+	waitpid(pid, NULL, 0);
+
+	return 0;
+}
