@@ -5,7 +5,7 @@
  * 
  * Part:        cfreader.c include file.
  *  
- * Version:     $Id: parser.h,v 1.1.12 2006/03/09 01:22:13 acassen Exp $
+ * Version:     $Id: parser.h,v 1.1.15 2007/09/15 04:07:41 acassen Exp $
  *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
  *
@@ -19,7 +19,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2006 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2007 Alexandre Cassen, <acassen@freebox.fr>
  */
 
 #ifndef _PARSER_H
@@ -55,7 +55,7 @@ struct keyword {
 
 /* global vars exported */
 extern vector keywords;
-extern FILE *stream;
+extern FILE *current_stream;
 extern int reload;
 
 /* Prototypes */
